#!/usr/bin/env python3
# Copyright 2024 Johannes Schauer Marin Rodrigues <josch@mister-muffin.de>
# SPDX-License-Identifier: MIT
# inspired by https://gitlab.com/carbonOS/gde/gde-background

import gi


gi.require_version("Gtk", "3.0")
gi.require_version("GtkLayerShell", "0.1")
gi.require_version("GDesktopEnums", "3.0")

from gi.repository import Gtk, Gdk, GtkLayerShell, GDesktopEnums, Gio, GdkPixbuf, GLib


class BackgroundWindow(Gtk.Window):
    def __init__(self):
        super().__init__()
        self.set_decorated(False)
        self.set_title("reform-wallpaper")
        self._pixbuf = GdkPixbuf.Pixbuf.new(
            GdkPixbuf.Colorspace.RGB, False, 8, 200, 200
        )
        self._file_monitor = None
        self.connect("draw", self._draw_cb)
        self.connect("size-allocate", self._size_allocate_cb)
        GtkLayerShell.init_for_window(self)
        GtkLayerShell.set_exclusive_zone(self, -1)
        GtkLayerShell.set_layer(self, GtkLayerShell.Layer.BACKGROUND)
        GtkLayerShell.set_anchor(self, GtkLayerShell.Edge.LEFT, True)
        GtkLayerShell.set_anchor(self, GtkLayerShell.Edge.RIGHT, True)
        GtkLayerShell.set_anchor(self, GtkLayerShell.Edge.TOP, True)
        GtkLayerShell.set_anchor(self, GtkLayerShell.Edge.BOTTOM, True)
        self._settings = Gio.Settings(schema_id="org.gnome.desktop.background")
        self._settings.connect("changed", self._settings_changed_cb)

    def _settings_changed_cb(self, settings, key):
        if key in [
            "picture-uri",
            "primary-color",
            "picture-options",
            "picture-opacity",
        ]:
            self._size_allocate_cb(self, self.get_size())

    def _draw_cb(self, widget, ctx):
        ctx.save()
        ctx.scale(1.0 / self.get_scale_factor(), 1.0 / self.get_scale_factor())
        Gdk.cairo_set_source_pixbuf(ctx, self._pixbuf, 0, 0)
        ctx.paint()
        ctx.restore()
        return True

    def _size_allocate_cb(self, widget, rect):
        self._pixbuf = GdkPixbuf.Pixbuf.new(
            GdkPixbuf.Colorspace.RGB,
            False,
            8,
            rect.width * self.get_scale_factor(),
            rect.height * self.get_scale_factor(),
        )
        fill_color = Gdk.RGBA()
        if fill_color.parse(self._settings.get_string("primary-color")):
            self._pixbuf.fill(
                (
                    (int(fill_color.red * 0xFF)) << 24
                    | (int(fill_color.green * 0xFF)) << 16
                    | (int(fill_color.blue * 0xFF)) << 8
                )
                + 0xFF
            )
        else:
            # if parsing the value failed, fill with the default color from
            # /usr/share/glib-2.0/schemas/org.gnome.desktop.background.gschema.xml
            self._pixbuf.fill(0x023C88FF)

        alpha = 255.0 * (self._settings.get_int("picture-opacity") / 100.0)
        if alpha == 0:
            self.queue_draw()
            return

        bg_mode = self._settings.get_enum("picture-options")
        match bg_mode:
            case GDesktopEnums.BackgroundStyle.NONE:
                self.queue_draw()
                return
            case GDesktopEnums.BackgroundStyle.SCALED:
                pass
            case GDesktopEnums.BackgroundStyle.ZOOM:
                pass
            case _:
                print("W: background style not supported:", bg_mode)

        path = self._settings.get_string("picture-uri")
        scheme = GLib.uri_parse_scheme(path)
        if scheme == "file":
            path = GLib.filename_from_uri(path)[0]
        else:
            print(f"W: scheme {scheme} not supported")
        if path.endswith(".xml"):
            print("W: GNOME timed wallpaper xml not supported")
        if self._file_monitor is not None:
            self._file_monitor.cancel()
            self._file_monitor = None
        try:
            self._file_monitor = Gio.File.new_for_path(path).monitor(
                Gio.FileMonitorFlags.NONE
            )
            self._file_monitor.connect(
                "changed",
                lambda a, b, c, d: self._size_allocate_cb(self, self.get_size()),
            )
        except Exception as e:
            print(f"E: attaching the file monitor failed with: {e}")
        img = GdkPixbuf.Pixbuf.new_from_file(path)
        ratio_w = self._pixbuf.get_width() / img.get_width()
        ratio_h = self._pixbuf.get_height() / img.get_height()
        if bg_mode == GDesktopEnums.BackgroundStyle.SCALED:
            ratio = ratio_w if ratio_w < ratio_h else ratio_h
        else:
            ratio = ratio_w if ratio_w > ratio_h else ratio_h
        final_width = ratio * img.get_width()
        final_height = ratio * img.get_height()
        off_x = (self._pixbuf.get_width() - final_width) / 2
        off_y = (self._pixbuf.get_height() - final_height) / 2
        if bg_mode == GDesktopEnums.BackgroundStyle.SCALED:
            img.composite(
                self._pixbuf,
                off_x,
                off_y,
                final_width,
                final_height,
                off_x,
                off_y,
                ratio,
                ratio,
                GdkPixbuf.InterpType.BILINEAR,
                alpha,
            )
        else:
            img.composite(
                self._pixbuf,
                0,
                0,
                self._pixbuf.get_width(),
                self._pixbuf.get_height(),
                off_x,
                off_y,
                ratio,
                ratio,
                GdkPixbuf.InterpType.BILINEAR,
                alpha,
            )
        self.queue_draw()


def create_window(display, monitor):
    win = BackgroundWindow()
    GtkLayerShell.set_monitor(win, monitor)
    setattr(monitor, "reform-wallpaper", win)
    win.show_all()


def destroy_window(display, monitor):
    getattr(monitor, "reform-wallpaper").destroy()


if __name__ == "__main__":
    display = Gdk.Display.get_default()
    for i in range(display.get_n_monitors()):
        create_window(display, display.get_monitor(i))
    display.connect("monitor-added", create_window)
    display.connect("monitor-removed", destroy_window)

    Gtk.main()



#!/usr/bin/env python3
# Copyright 2025 Giraut
# Copyright 2026 Johannes Schauer Marin Rodrigues
# SPDX-License-Identifier: MIT
#
# https://github.com/Giraut/mnt_reform_keyboard_backlight_rainbow


### Constants
keycap_widths = {
    1: 17.5,
    1.25: 17.5 * 1.25 + 0.1,  # 21.975
    1.5: 17.5 * 1.5 + 0.25,  # 26.5
    1.75: 31.3,
    2: 17.5 * 2 + 0.5,  # 35.5
}

classic_kbd_width = 260.65
classic_sizes = [
    [1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1.5],
    [1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1.5],
    [1.5, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1],
    [1.75, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1.75],
    [1.25, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1.25],
    [1.25, 1.5, 1.5, 1.5, 2, 1.5, 1, 1, 1, 1, 1.25],
]

# ortholinear pocket reform
# yes, the pocket keyboard only has 5 rows and 12 columns but we need to send
# the full matrix of the big keyboard
pocket_sizes = [[1] * 14] * 6
# this is not the real pocket keyboard width but we make our live easier by
# re-using the default keycap sizes this way
pocket_kbd_width = 251.65


### Parameters
default_backlight_intensity = 50  # %

### Modules
import sys
import argparse
from time import time, sleep
import glob

try:
    from setproctitle import setproctitle
except ModuleNotFoundError:
    setproctitle = lambda a: True


### Routines
def flatten(xss):
    return [x for xs in xss for x in xs]


def hsv_to_bgr(h: float, s: float, v: float) -> list:
    if not s:
        return [v, v, v]
    if h == 1.0:
        h = 0.0
    i = int(h * 6.0)
    f = h * 6.0 - i

    w = int(v * (1.0 - s) * 255)
    q = int(v * (1.0 - s * f) * 255)
    t = int(v * (1.0 - s * (1.0 - f)) * 255)
    v = int(255 * v)

    match i:
        case 0:
            return [w, t, v]
        case 1:
            return [w, v, q]
        case 2:
            return [t, v, w]
        case 3:
            return [v, q, w]
        case 4:
            return [v, w, t]
        case 5:
            return [q, w, v]


def argparse_intensity_type(value):
    """Argparse type for a valid backlight intensity"""

    try:
        v = float(value)
    except:
        raise argparse.ArgumentTypeError("{}: invalid intensity".format(value))

    if v < 0:
        raise argparse.ArgumentTypeError("{}: intensity too low (min. 0%)".format(v))

    if v > 100:
        raise argparse.ArgumentTypeError("{}: intensity too high (max. 100%)".format(v))

    return v


def argparse_delay_type(value):
    """Argparse type for a valid backlight refresh delay"""

    try:
        v = float(value)
    except:
        raise argparse.ArgumentTypeError("{}: invalid delay".format(value))

    if v < 1:
        raise argparse.ArgumentTypeError("{}: delay too low (min. 1s)".format(v))

    return v


### Main routine
def main():
    setproctitle("mnt_reform_keyboard_backlight_rainbow")

    # Process the command line arguments
    argparser = argparse.ArgumentParser()

    argparser.add_argument(
        "-i",
        "--intensity",
        type=argparse_intensity_type,
        help="Backlight intensity (default: {}%%)".format(default_backlight_intensity),
        default=default_backlight_intensity,
    )

    argparser.add_argument(
        "-r",
        "--refresh-every-sec",
        type=argparse_delay_type,
        help="Number of seconds between refreshes "
        "(default: no refresh - set the pattern and exit)",
    )

    args = argparser.parse_args()

    mnt_keyboard4_hidraw_device = None
    # try hidraw devices until one matches
    for name, sizes, kbd_width, g in [
        (
            "MNT Keyboard v4",
            classic_sizes,
            classic_kbd_width,
            "/dev/input/by-id/usb-MNT_Research_MNT_Reform_Keyboard_4.0_*-hidraw",
        ),
        (
            "MNT Pocket Reform Keyboard",
            pocket_sizes,
            pocket_kbd_width,
            "/dev/input/by-id/usb-MNT_Pocket_Reform_Input_1.0_*-hidraw",
        ),
    ]:
        matches = glob.glob(g)
        if len(matches) != 1:
            continue
        mnt_keyboard4_hidraw_device = matches[0]
        print(f"Found {name}")
        break

    if mnt_keyboard4_hidraw_device is None:
        print("unable to find hidraw device")
        exit(1)

    offsets = []
    for row in sizes:
        row_offsets = []
        offset_x = -17.5 / 2
        for u in row:
            offset_x += keycap_widths[u] / 2
            row_offsets.append(round(offset_x, 3))
            offset_x += keycap_widths[u] / 2
            offset_x += 1.1
        assert 0 <= offset_x <= kbd_width, offset_x
        offsets.append(row_offsets)

    max_num_retries = 5

    # Set the color of the backlight LEDs then exit, or run the animation
    while True:
        # Set the LEDs
        num_steps = 1000
        for seq in range(num_steps):
            start_backlight_change_tstamp = time()

            num_retries = 0
            row = 0
            while row < len(offsets):
                row_offsets = offsets[row]
                data = b"xXRGB" + bytes(
                    [row]
                    + flatten(
                        hsv_to_bgr(
                            (((offset - 17.5 / 4 * row) / kbd_width) + seq / num_steps)
                            % 1,
                            1.0,
                            args.intensity / 100,
                        )
                        for offset in row_offsets
                    )
                )
                try:
                    with open(mnt_keyboard4_hidraw_device, "wb") as k:
                        k.write(data)
                    row += 1
                    num_retries = 0
                except PermissionError:
                    print(
                        f"Unable to open {mnt_keyboard4_hidraw_device} for writing: permission denied"
                    )
                    exit(1)
                except BrokenPipeError:
                    print(
                        f"Unable to write to {mnt_keyboard4_hidraw_device}: broken pipe -- wrong device?"
                    )
                    num_retries += 1
                    if num_retries > max_num_retries:
                        print("failed too often")
                        exit(1)

                sleep(0.15)  # Give the keyboard controller a chance to do its thing

            # Exit if we're not doing an animation
            if args.refresh_every_sec is None:
                return 0

            # Sleep until the next refresh
            sleep(
                max(
                    0, args.refresh_every_sec - (time() - start_backlight_change_tstamp)
                )
            )


### Main program
if __name__ == "__main__":
    sys.exit(main())



#!/usr/bin/env python3
# Copyright 2023 - 2024 Lukas F. Hartmann <lukas@mntre.com>
# Copyright 2024 Johannes Schauer Marin Rodrigues <josch@debian.org>
# SPDX-License-Identifier: GPL-3.0+

import signal
import os
import subprocess
import gi
import shutil

gi.require_version("Gtk", "3.0")
gi.require_version("Notify", "0.7")
gi.require_version("AyatanaAppIndicator3", "0.1")
from gi.repository import Gtk
from gi.repository import Gdk
from gi.repository import Notify
from gi.repository import AyatanaAppIndicator3 as AppIndicator3

APPID = "reform-tray"
ICON = "view-more-symbolic"


def handle_exit_item(question, command):
    dialog = Gtk.MessageDialog(
        flags=0,
        message_type=Gtk.MessageType.WARNING,
        text=question,
    )
    dialog.format_secondary_text("Make sure to save your work first.")
    dialog.add_buttons(
        Gtk.STOCK_OK,
        Gtk.ResponseType.OK,
        Gtk.STOCK_CANCEL,
        Gtk.ResponseType.CANCEL,
    )
    response = dialog.run()

    if response == Gtk.ResponseType.OK:
        subprocess.run(command)

    dialog.destroy()


class MenuItem(Gtk.ImageMenuItem):
    def __init__(self, label, icon, args, exitmsg=None):
        Gtk.ImageMenuItem.__init__(self, label=label, visible=True)
        self.img = Gtk.Image()
        self.img.set_from_icon_name(icon, -1)
        self.set_image(self.img)
        self.set_always_show_image(True)
        if exitmsg:
            self.action = lambda _: handle_exit_item(exitmsg, args)
        else:
            self.action = lambda _: subprocess.run(args)
        self.connect("activate", self.action)
        if not shutil.which(args[0]):
            self.set_sensitive(False)


menuitems = [
    MenuItem(
        "Help",
        "help-about-symbolic",
        ["xdg-terminal-exec", "bash", "-c", "reform-help --desktop; bash"],
        None,
    ),
    MenuItem(
        "Higher Brightness",
        "weather-clear-symbolic",
        ["brightnessctl", "set", "10%+"],
        None,
    ),
    MenuItem(
        "Lower Brightness",
        "weather-clear-night-symbolic",
        ["brightnessctl", "set", "10%-"],
        None,
    ),
    MenuItem("Terminal", "utilities-terminal-symbolic", ["xdg-terminal-exec"], None),
    MenuItem("File Manager", "folder-symbolic", ["xdg-open", os.environ["HOME"]], None),
    MenuItem(
        "System Monitor",
        "utilities-system-monitor-symbolic",
        ["gnome-system-monitor"],
        None,
    ),
    MenuItem(
        "Logout",
        "system-log-out-symbolic",
        ["pkill", "(wayfire|sway)"],
        "Are you sure you want to exit the desktop session?",
    ),
    MenuItem(
        "Reboot",
        "system-restart-symbolic",
        ["systemctl", "reboot"],
        "Are you sure you want to reboot the computer?",
    ),
    MenuItem(
        "Shutdown",
        "system-shutdown-symbolic",
        ["systemctl", "poweroff"],
        "Are you sure you want to shut down the computer?",
    ),
]


class TrayIcon:
    def __init__(self, appid, icon, menuitems):
        self.menu = Gtk.Menu()
        for mi in menuitems:
            self.menu.append(mi)

        self.ind = AppIndicator3.Indicator.new(
            appid, icon, AppIndicator3.IndicatorCategory.HARDWARE
        )
        self.ind.set_status(AppIndicator3.IndicatorStatus.ACTIVE)
        self.ind.set_secondary_activate_target(mi)
        self.ind.set_menu(self.menu)


signal.signal(signal.SIGINT, signal.SIG_DFL)
icon = TrayIcon(APPID, ICON, menuitems)
Notify.init(APPID)

Gtk.main()
